import {
    createCheckoutService,
    Address,
    CheckoutRequestBody,
    CheckoutSelectors,
    CheckoutService,
    Country,
    Customer,
    FormField,
} from '@bigcommerce/checkout-sdk';
import { noop } from 'lodash';
import React, { Component, ReactNode } from 'react';
import { connect } from 'react-redux'

import Swal from 'sweetalert2'
import axios from 'axios'

import B3Checkout from '../b3checkout'
import { AddressFormSkeleton } from '@bigcommerce/checkout/ui';

import { isEqualAddress, mapAddressFromFormValues } from '../address';
import { CheckoutContextProps, withCheckout } from '../checkout';
import { EMPTY_ARRAY, isFloatingLabelEnabled } from '../common/utility';
import { TranslatedString } from '../locale';
import { getShippableItemsCount } from '../shipping';
import { Legend } from '../ui/form';

import BillingForm, { BillingFormValues } from './BillingForm';
import CheckoutStepType from '../checkout/CheckoutStepType';
import {  NoAddressTip } from '../b3checkout'
import getBillingMethodId from './getBillingMethodId';

export interface BillingProps {
    navigateNextStep(): void;
    onReady?(): void;
    onUnhandledError(error: Error): void;
    type: CheckoutStepType
}

export interface WithCheckoutBillingProps {
    countries: Country[];
    countriesWithAutocomplete: string[];
    customer: Customer;
    customerMessage: string;
    googleMapsApiKey: string;
    isInitializing: boolean;
    isUpdating: boolean;
    shouldShowOrderComments: boolean;
    billingAddress?: Address;
    methodId?: string;
    useFloatingLabel?: boolean;
    getFields(countryCode?: string): FormField[];
    initialize(): Promise<CheckoutSelectors>;
    updateAddress(address: Partial<Address>): Promise<CheckoutSelectors>;
    updateCheckout(payload: CheckoutRequestBody): Promise<CheckoutSelectors>;
}

interface B3Props {
    isB3User: boolean
    b3Company: any
    checkoutSettings: any
}

class Billing extends Component<B3Props & BillingProps & WithCheckoutBillingProps> {
    async componentDidMount(): Promise<void> {
        const { initialize, onReady = noop, onUnhandledError } = this.props;

        try {
            await initialize();
            onReady();
        } catch (error) {
            if (error instanceof Error) {
                onUnhandledError(error);
            }
        }
    }

    get shouldAllowNewAddress() {
        const { checkoutSettings } = this.props

        return checkoutSettings['billing.address.add']
    }

    get addresses() {
        const {
            customer,
            isB3User,
            b3Company,
        } = this.props

        return isB3User
            ? b3Company.addresses.filter((address: { isBilling: number }) => address.isBilling === 1)
            : customer.addresses
    }

    render(): ReactNode {
        const {
            updateAddress,
            isInitializing,
            type,
            isB3User,
            ...props
        } = this.props;

        const isShouldShowNewAddressForm = !(isB3User ? !this.addresses.length && !this.shouldAllowNewAddress : false)

        return (
            <AddressFormSkeleton isLoading={isInitializing}>
                <div className="checkout-form">
                    {
                        isShouldShowNewAddressForm  ? (
                            <>
                                <div className="form-legend-container">
                                    <Legend testId="billing-address-heading">
                                        <TranslatedString id="billing.billing_address_heading" />
                                    </Legend>
                                </div>
                                <BillingForm
                                    {...props}
                                    onSubmit={this.handleSubmit}
                                    type={ type }
                                    updateAddress={updateAddress}
                                />
                            </>
                        ) : (<NoAddressTip type="billing" />)
                    }
                    
                </div>
            </AddressFormSkeleton>
        );
    }

    private handleSubmit: (values: BillingFormValues) => void = async ({
        orderComment,
        ...addressValues
    }) => {
        const {
            updateAddress,
            updateCheckout,
            customerMessage,
            billingAddress,
            navigateNextStep,
            onUnhandledError,
            isB3User,
            b3Company,
            getFields,
            checkoutSettings,
        } = this.props;

        const promises: Array<Promise<CheckoutSelectors>> = [];
        const address = mapAddressFromFormValues(addressValues);

        let checkoutService: CheckoutService;
        checkoutService = createCheckoutService();
        const state = await checkoutService.loadCheckout();
        const config = state.data.getConfig();
        const cart = state.data.getCart();
        const currencyCode = config?.currency?.code;
        const items = cart?.lineItems;

        if (items?.digitalItems.length && !items?.physicalItems.length && !items?.customItems?.length && !items?.giftCertificates?.length) {
            if (address.countryCode == 'CA') {
                // set canada
                if (currencyCode == 'USD') {
                    axios.post('/cart/change-currency', {
                        currencyCode: "CAD"
                    })
                    .then(async () => {
                        await Swal.fire({
                            icon: 'warning',
                            text: "Looks like you are shipping to Canada address. Page will reload to apply Canada dollar currency.",
                            allowOutsideClick: false,
                            allowEscapeKey: false,
                        })
                        window.location.reload();
                    })
                    .catch(function (error) {
                        console.error(error);
                    });
                }
            } else {
                // set usd
                if (currencyCode == 'CAD') {
                    axios.post('/cart/change-currency', {
                        currencyCode: "USD"
                    })
                    .then(async () => {
                        await Swal.fire({
                            icon: 'warning',
                            text: "Looks like you are not shipping to Canada address. Page will reload to apply United States dollar currency.",
                            allowOutsideClick: false,
                            allowEscapeKey: false,
                        })
                        window.location.reload();
                    })
                    .catch(function (error) {
                        console.error(error);
                    });
                }
            }
        }

        if (isB3User && (addressValues as any)?.b2bCustomFields) {
            B3Checkout.extrasFiles.saveExtrasAddress((addressValues as any)?.b2bCustomFields, 'billing')
        }
        if (isB3User && checkoutSettings['billing.address.save']) {
            const formFields = getFields(address?.countryCode)
            const isValid = B3Checkout.utils.util.validateAddress(address as Address, b3Company.addresses, formFields)
            await B3Checkout.addresses.addNewB3Address({
                newAddress: {
                    ...address,
                    addressId: isValid ? undefined : ''
                },
                isBillingMode: true,
                checkoutSettings,
                b2bCustomFields: B3Checkout.extrasFiles.decodeAddressExtraFieldsKeys((addressValues as any)?.b2bCustomFields)
            })
        }

        if (address && !isEqualAddress(address, billingAddress)) {
            promises.push(updateAddress(address));
        }

        if (customerMessage !== orderComment) {
            promises.push(updateCheckout({ customerMessage: orderComment }));
        }

        try {
            await Promise.all(promises);

            navigateNextStep();
        } catch (error) {
            if (error instanceof Error) {
                onUnhandledError(error);
            }
        }
    };
}

function mapToBillingProps({
    checkoutService,
    checkoutState,
}: CheckoutContextProps): WithCheckoutBillingProps | null {
    const {
        data: {
            getCheckout,
            getConfig,
            getCart,
            getCustomer,
            getBillingAddress,
            getBillingAddressFields,
            getBillingCountries,
        },
        statuses: { isLoadingBillingCountries, isUpdatingBillingAddress, isUpdatingCheckout },
    } = checkoutState;

    const config = getConfig();
    const customer = getCustomer();
    const checkout = getCheckout();
    const cart = getCart();

    if (!config || !customer || !checkout || !cart) {
        return null;
    }

    const { enableOrderComments, googleMapsApiKey, features } = config.checkoutSettings;

    const countriesWithAutocomplete = ['US', 'CA', 'AU', 'NZ'];

    if (features['CHECKOUT-4183.checkout_google_address_autocomplete_uk']) {
        countriesWithAutocomplete.push('GB');
    }

    return {
        billingAddress: getBillingAddress(),
        countries: getBillingCountries() || EMPTY_ARRAY,
        countriesWithAutocomplete,
        customer,
        customerMessage: checkout.customerMessage,
        getFields: getBillingAddressFields,
        googleMapsApiKey,
        initialize: checkoutService.loadBillingAddressFields,
        isInitializing: isLoadingBillingCountries(),
        isUpdating: isUpdatingBillingAddress() || isUpdatingCheckout(),
        methodId: getBillingMethodId(checkout),
        shouldShowOrderComments: enableOrderComments && getShippableItemsCount(cart) < 1,
        updateAddress: checkoutService.updateBillingAddress,
        updateCheckout: checkoutService.updateCheckout,
        useFloatingLabel: isFloatingLabelEnabled(config.checkoutSettings),
    };
}

const mapStateToProps = (state: { global: { isB3User: boolean, checkoutSettings: any }, company: any }) => {
    const {
        global: {
            isB3User,
            checkoutSettings,
        },
        company: b3Company,

    } = state

    return {
        isB3User,
        b3Company,
        checkoutSettings,
    }
}

export default connect(mapStateToProps)(withCheckout(mapToBillingProps)(Billing));
